@@ -69,8 +69,7 @@ class QHELP_EXPORT QHelpSearchEngine : public QObject
     Q_OBJECT
 
 public:
-    explicit QHelpSearchEngine(QHelpEngineCore *helpEngine,
-        QObject *parent = 0);
+    explicit QHelpSearchEngine(QHelpEngineCore *helpEngine, QObject *parent = Q_NULLPTR);
     ~QHelpSearchEngine();
 
     QHelpSearchQueryWidget* queryWidget();
