@@ -33,16 +33,15 @@
 
 #include "qdbusviewer.h"
 #include "qdbusmodel.h"
+#include "servicesproxymodel.h"
 #include "propertydialog.h"
 #include "logviewer.h"
 
-#include <QtWidgets/QTreeWidget>
+
 #include <QtCore/QStringListModel>
-#include <QtCore/QSortFilterProxyModel>
 #include <QtCore/QMetaProperty>
 #include <QtCore/QSettings>
 #include <QtWidgets/QLineEdit>
-#include <QtWidgets/QListView>
 #include <QtWidgets/QAction>
 #include <QtWidgets/QShortcut>
 #include <QtWidgets/QVBoxLayout>
@@ -50,6 +49,9 @@
 #include <QtWidgets/QInputDialog>
 #include <QtWidgets/QMessageBox>
 #include <QtWidgets/QMenu>
+#include <QtWidgets/QTableWidget>
+#include <QtWidgets/QTreeWidget>
+#include <QtWidgets/QHeaderView>
 #include <QtDBus/QDBusConnectionInterface>
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusMetaType>
@@ -79,16 +81,27 @@ QDBusViewer::QDBusViewer(const QDBusConnection &connection, QWidget *parent)  :
     c(connection),
     objectPathRegExp(QLatin1String("\\[ObjectPath: (.*)\\]"))
 {
-    servicesModel = new QStringListModel(this);
-    servicesFilterModel = new QSortFilterProxyModel(this);
-    servicesFilterModel->setSourceModel(servicesModel);
-    servicesFilterModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
     serviceFilterLine = new QLineEdit(this);
     serviceFilterLine->setPlaceholderText(tr("Search..."));
-    servicesView = new QListView(this);
-    servicesView->setModel(servicesFilterModel);
 
-    connect(serviceFilterLine, SIGNAL(textChanged(QString)), servicesFilterModel, SLOT(setFilterFixedString(QString)));
+    // Create model for services list
+    servicesModel = new QStringListModel(this);
+    // Wrap service list model in proxy for easy filtering and interactive sorting
+    servicesProxyModel = new ServicesProxyModel(this);
+    servicesProxyModel->setSourceModel(servicesModel);
+    servicesProxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
+
+    servicesView = new QTableView(this);
+    servicesView->setModel(servicesProxyModel);
+    // Make services grid view behave like a list view with headers
+    servicesView->verticalHeader()->hide();
+    servicesView->horizontalHeader()->setStretchLastSection(true);
+    servicesView->setShowGrid(false);
+    // Sort service list by default
+    servicesView->setSortingEnabled(true);
+    servicesView->sortByColumn(0, Qt::AscendingOrder);
+
+    connect(serviceFilterLine, SIGNAL(textChanged(QString)), servicesProxyModel, SLOT(setFilterFixedString(QString)));
 
     tree = new QTreeView;
     tree->setContextMenuPolicy(Qt::CustomContextMenu);
@@ -432,6 +445,8 @@ void QDBusViewer::dumpMessage(const QDBusMessage &message)
             QString str = QDBusUtil::argumentToString(arg).toHtmlEscaped();
             // turn object paths into clickable links
             str.replace(objectPathRegExp, QLatin1String("[ObjectPath: <a href=\"qdbus://bus\\1\">\\1</a>]"));
+            // convert new lines from command to proper HTML line breaks
+            str.replace(QStringLiteral("\n"), QStringLiteral("<br/>"));
             out += str;
             out += QLatin1String(", ");
         }
